RaidIconBarData = {};

RaidIconBarData.scale = 1;
RaidIconBarData.border = false;
RaidIconBarData.layout = nil;
RaidIconBarData.flares = nil;
RaidIconBarData.ribVisible = true;
RaidIconBarData.flaresVisible = true;

-- Bindings
BINDING_HEADER_RIBHEADER = "RaidIconBar Snap";
BINDING_NAME_RIB_ICONSNAP = "RaidIcons";
BINDING_NAME_RIB_MARKERSNAP = "RaidMarker";

icons_posx = 0;
icons_posy = 0;
marker_posx = 0;
marker_posy = 0;

-- PlaceRaidMarker(arg1, arg2);
-- ClearRaidMarker(arg1);

local function initDefaults()
    RaidIconBarData.scale = RaidIconBarData.scale or 1;
    RaidIconBarData.layout = RaidIconBarData.layout or nil;
    RaidIconBarData.flares = RaidIconBarData.flares or nil;
    
    -- booleans
    if (RaidIconBarData.border == nil) then
        RaidIconBarData.border = false;
    end
    if (RaidIconBarData.ribVisible == nil) then
        RaidIconBarData.ribVisible = true;
    end
    if (RaidIconBarData.flaresVisible == nil) then
        RaidIconBarData.flaresVisible = true;
    end
end

-- functions
function RaidIconBar_OnLoad(self)
    self:RegisterEvent("ADDON_LOADED");

    SLASH_RAIDICONBAR1 = "/raidiconbar";
    SLASH_RAIDICONBAR2 = "/rib";
    SlashCmdList["RAIDICONBAR"] = function(msg)
        RaidIconBar_SlashCommandHandler(msg);
    end

    RaidIconBar_IconFrame:SetClampedToScreen(true);
end

function RaidIconBar_OnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then

        local addonName = ...;
        if (addonName == "RaidIconBar") then
            initDefaults();
            RaidIconBar_InitScale();
            RaidIconBar_Reinit();
            RaidIconBar_MarkerSnapUp();
        end
    end
end

function RaidIconBar_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    end

    if (msg == "border") then
        if (RaidIconBarData.border) then
            RaidIconBarData.border = false;
        else
            RaidIconBarData.border = true;
        end
    elseif (msg == "reset") then
        RaidIconBarData.scale = 1;
        RaidIconBarData.border = false;
        RaidIconBarData.layout = nil;
        RaidIconBarData.flares = nil;
        RaidIconBarData.ribVisible = true;
        RaidIconBarData.flaresVisible = true;
    elseif (msg == "toggle icons") then
        if (RaidIconBarData.ribVisible) then
            RaidIconBarData.ribVisible = false;
        else
            RaidIconBarData.ribVisible = true;
        end
    elseif (msg == "toggle flares") then
        if (RaidIconBarData.flaresVisible) then
            RaidIconBarData.flaresVisible = false;
        else
            RaidIconBarData.flaresVisible = true;
        end
    elseif (string.find(msg, "layout") == 1) then
        local str_index = string.sub(msg, 8);
        local index = tonumber(str_index);

        if (index and index >= 1 and index <= 9) then
            RaidIconBarData.layout = index;
            DEFAULT_CHAT_FRAME:AddMessage("RaidIconBar: set layout to "..RaidIconBarData.layout, 1.0, 1.0, 1.0, 1, 10);
        end
    elseif (string.find(msg, "flares") == 1) then
        local str_index = string.sub(msg, 8);
        local index = tonumber(str_index);

        if (index and index >= 1 and index <= 3) then
            RaidIconBarData.flares = index;
            DEFAULT_CHAT_FRAME:AddMessage("RaidIconBar: set flares layout to "..RaidIconBarData.flares, 1.0, 1.0, 1.0, 1, 10);
        end
    elseif (string.find(msg, "scale") == 1) then
        local str_scale = string.sub(msg, 7);
        local scale = tonumber(str_scale);

        if (scale and scale >= 0.1 and scale <= 4) then
            RaidIconBarData.scale = scale;
            RaidIconBar_InitScale();
        end
    else
        DEFAULT_CHAT_FRAME:AddMessage("RaidIconBar:", 0.3, 0.3, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib reset|r            reset settings", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib reload|r           reload UI", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib border|r           toggle border", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib toggle icons|r     toggle raid icons", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib toggle flares|r    toggle raid flares", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib layout #|r         1 - 9    change icon layout", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib flares #|r         1 - 3    change flares layout", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff/rib scale #|r          0.1 - 4.0    scale buttons", 1.0, 1.0, 1.0, 1, 10);
    end

    RaidIconBar_Reinit();
end

function RaidIconBar_InitScale()

    if (RaidIconBarData.scale == nil) then
        RaidIconBarData.scale = 1;
    end

    getglobal("RaidIconBar_IconFrame"):SetScale(RaidIconBarData.scale);
    getglobal("RaidIconBar_MarkerFrame"):SetScale(RaidIconBarData.scale);
end

function RaidIconBar_IconsOnMouseDown(frame, button)
    if (button == "RightButton") then
        if (IsControlKeyDown()) then
            RaidIconBar_IconFrame:StartMoving();
        end
    end
end

function RaidIconBar_IconsOnMouseUp(frame, button)
    if (button == "RightButton") then
        RaidIconBar_IconFrame:StopMovingOrSizing();
    end
end

function RaidIconBar_MarkerOnMouseDown(frame, button)
    if (button == "RightButton") then
        if (IsControlKeyDown()) then
            RaidIconBar_MarkerFrame:StartMoving();
        end
    end
end

function RaidIconBar_MarkerOnMouseUp(frame, button)
    if (button == "RightButton") then
        RaidIconBar_MarkerFrame:StopMovingOrSizing();
    end
end

function RaidIconBar_OnClick(self, button, down)
    local id = self:GetID();

    if (id == 9) then
        DoReadyCheck();
    else
        if (button == "LeftButton") then
            SetRaidTarget("target", id);
        elseif (button == "RightButton") then
            SetRaidTarget("target", 0);
        end
    end
end

function RaidIconBar_ResetLayout()
    RaidIconBar_IconFrame:SetWidth(230);
    RaidIconBar_IconFrame:SetHeight(40);

    RaidIconBar_IconButton1:ClearAllPoints();
    RaidIconBar_IconButton1:SetPoint("CENTER", -94, 0);
    RaidIconBar_IconButton2:ClearAllPoints();
    RaidIconBar_IconButton2:SetPoint("CENTER", -67, 0);
    RaidIconBar_IconButton3:ClearAllPoints();
    RaidIconBar_IconButton3:SetPoint("CENTER", -40, 0);
    RaidIconBar_IconButton4:ClearAllPoints();
    RaidIconBar_IconButton4:SetPoint("CENTER", -13, 0);
    RaidIconBar_IconButton5:ClearAllPoints();
    RaidIconBar_IconButton5:SetPoint("CENTER", 13, 0);
    RaidIconBar_IconButton6:ClearAllPoints();
    RaidIconBar_IconButton6:SetPoint("CENTER", 40, 0);
    RaidIconBar_IconButton7:ClearAllPoints();
    RaidIconBar_IconButton7:SetPoint("CENTER", 67, 0);
    RaidIconBar_IconButton8:ClearAllPoints();
    RaidIconBar_IconButton8:SetPoint("CENTER", 94, 0);
    RaidIconBar_IconButton9:ClearAllPoints();
    RaidIconBar_IconButton9:SetPoint("CENTER", 130, 0);

    RaidIconBarData.layout = nil;
end

function RaidIconBar_Reinit()
    if (RaidIconBarData.ribVisible) then
        RaidIconBar_IconFrame:Show();
    else
        RaidIconBar_IconFrame:Hide();
    end
    
    if (RaidIconBarData.flaresVisible) then
        RaidIconBar_MarkerFrame:Show();
    else
        RaidIconBar_MarkerFrame:Hide();
    end

    if (RaidIconBarData.border) then
        RaidIconBar_IconFrame:SetBackdrop( {
            bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16,
            insets = { left = 4, right = 4, top = 4, bottom = 4 }
        });
    else
        RaidIconBar_IconFrame:SetBackdrop(nil);
    end

    RaidIconBar_LoadLayout(RaidIconBarData.layout);
    RaidIconBar_LoadFlaresLayout(RaidIconBarData.flares);
end

function RaidIconBar_LoadFlaresLayout(index)
    if (not index or index < 1 or index > 3) then
        index = 1;
    end

    RaidIconBar_MarkerButtonClear:ClearAllPoints();
    RaidIconBar_MarkerButtonBlue:ClearAllPoints();
    RaidIconBar_MarkerButtonGreen:ClearAllPoints();
    RaidIconBar_MarkerButtonPurple:ClearAllPoints();
    RaidIconBar_MarkerButtonRed:ClearAllPoints();
    RaidIconBar_MarkerButtonYellow:ClearAllPoints();
    RaidIconBar_MarkerButtonOrange:ClearAllPoints();
    RaidIconBar_MarkerButtonSilver:ClearAllPoints();
    RaidIconBar_MarkerButtonWhite:ClearAllPoints();
    
    if (index == 1) then
        RaidIconBar_MarkerButtonClear:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 0, 0);
        RaidIconBar_MarkerButtonBlue:SetPoint("RIGHT", RaidIconBar_MarkerButtonClear, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonGreen:SetPoint("RIGHT", RaidIconBar_MarkerButtonBlue, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonPurple:SetPoint("RIGHT", RaidIconBar_MarkerButtonGreen, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonRed:SetPoint("RIGHT", RaidIconBar_MarkerButtonPurple, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonYellow:SetPoint("RIGHT", RaidIconBar_MarkerButtonRed, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonOrange:SetPoint("RIGHT", RaidIconBar_MarkerButtonYellow, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonSilver:SetPoint("RIGHT", RaidIconBar_MarkerButtonOrange, "LEFT", 0, 0);
        RaidIconBar_MarkerButtonWhite:SetPoint("RIGHT", RaidIconBar_MarkerButtonSilver, "LEFT", 0, 0);
    elseif (index == 2) then
        RaidIconBar_MarkerButtonClear:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 0, 0);
        RaidIconBar_MarkerButtonBlue:SetPoint("TOP", RaidIconBar_MarkerButtonClear, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonGreen:SetPoint("TOP", RaidIconBar_MarkerButtonBlue, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonPurple:SetPoint("TOP", RaidIconBar_MarkerButtonGreen, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonRed:SetPoint("TOP", RaidIconBar_MarkerButtonPurple, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonYellow:SetPoint("TOP", RaidIconBar_MarkerButtonRed, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonOrange:SetPoint("TOP", RaidIconBar_MarkerButtonYellow, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonSilver:SetPoint("TOP", RaidIconBar_MarkerButtonOrange, "BOTTOM", 0, 0);
        RaidIconBar_MarkerButtonWhite:SetPoint("TOP", RaidIconBar_MarkerButtonSilver, "BOTTOM", 0, 0);
    elseif (index == 3) then
        RaidIconBar_MarkerButtonClear:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 0, 0);
        RaidIconBar_MarkerButtonBlue:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 0, 28);
        RaidIconBar_MarkerButtonGreen:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", -21, 20);
        RaidIconBar_MarkerButtonPurple:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 21, 20);
        RaidIconBar_MarkerButtonRed:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", -30, 0);
        RaidIconBar_MarkerButtonYellow:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 30, 0);
        RaidIconBar_MarkerButtonOrange:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 0, -28);
        RaidIconBar_MarkerButtonSilver:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", 21, -20);
        RaidIconBar_MarkerButtonWhite:SetPoint("CENTER", RaidIconBar_MarkerFrame, "CENTER", -21, -20);
    end
end

function RaidIconBar_LoadLayout(index)
    if (not index or index < 1 or index > 9) then
        index = 1;
    end

    RaidIconBar_IconButton1:ClearAllPoints();
    RaidIconBar_IconButton2:ClearAllPoints();
    RaidIconBar_IconButton3:ClearAllPoints();
    RaidIconBar_IconButton4:ClearAllPoints();
    RaidIconBar_IconButton5:ClearAllPoints();
    RaidIconBar_IconButton6:ClearAllPoints();
    RaidIconBar_IconButton7:ClearAllPoints();
    RaidIconBar_IconButton8:ClearAllPoints();
    RaidIconBar_IconButton9:ClearAllPoints();

    if (index == 1) then
        RaidIconBar_IconFrame:SetWidth(230);
        RaidIconBar_IconFrame:SetHeight(40);

        RaidIconBar_ResetLayout();
    elseif (index == 2) then
        RaidIconBar_IconFrame:SetWidth(230);
        RaidIconBar_IconFrame:SetHeight(40);

        RaidIconBar_IconButton1:SetPoint("CENTER", 94, 0);
        RaidIconBar_IconButton2:SetPoint("CENTER", 67, 0);
        RaidIconBar_IconButton3:SetPoint("CENTER", 40, 0);
        RaidIconBar_IconButton4:SetPoint("CENTER", 13, 0);
        RaidIconBar_IconButton5:SetPoint("CENTER", -13, 0);
        RaidIconBar_IconButton6:SetPoint("CENTER", -40, 0);
        RaidIconBar_IconButton7:SetPoint("CENTER", -67, 0);
        RaidIconBar_IconButton8:SetPoint("CENTER", -94, 0);

        RaidIconBar_IconButton9:SetPoint("CENTER", -130, 0);
    elseif (index == 3) then
        RaidIconBar_IconFrame:SetWidth(120);
        RaidIconBar_IconFrame:SetHeight(70);

        RaidIconBar_IconButton1:SetPoint("CENTER", -40, -13);
        RaidIconBar_IconButton2:SetPoint("CENTER", -13, -13);
        RaidIconBar_IconButton3:SetPoint("CENTER", 13, -13);
        RaidIconBar_IconButton4:SetPoint("CENTER", 40, -13);
        RaidIconBar_IconButton5:SetPoint("CENTER", -40, 13);
        RaidIconBar_IconButton6:SetPoint("CENTER", -13, 13);
        RaidIconBar_IconButton7:SetPoint("CENTER", 13, 13);
        RaidIconBar_IconButton8:SetPoint("CENTER", 40, 13);

        RaidIconBar_IconButton9:SetPoint("CENTER", 70, 0);
    elseif (index == 4) then
        RaidIconBar_IconFrame:SetWidth(120);
        RaidIconBar_IconFrame:SetHeight(70);

        RaidIconBar_IconButton1:SetPoint("CENTER", 40, -13);
        RaidIconBar_IconButton2:SetPoint("CENTER", 13, -13);
        RaidIconBar_IconButton3:SetPoint("CENTER", -13, -13);
        RaidIconBar_IconButton4:SetPoint("CENTER", -40, -13);
        RaidIconBar_IconButton5:SetPoint("CENTER", 40, 13);
        RaidIconBar_IconButton6:SetPoint("CENTER", 13, 13);
        RaidIconBar_IconButton7:SetPoint("CENTER", -13, 13);
        RaidIconBar_IconButton8:SetPoint("CENTER", -40, 13);

        RaidIconBar_IconButton9:SetPoint("CENTER", -70, 0);
    elseif (index == 5) then
        RaidIconBar_IconFrame:SetWidth(95);
        RaidIconBar_IconFrame:SetHeight(95);

        RaidIconBar_IconButton1:SetPoint("CENTER", 0, -27);
        RaidIconBar_IconButton2:SetPoint("CENTER", -27, -27);
        RaidIconBar_IconButton3:SetPoint("CENTER", 27, 0);
        RaidIconBar_IconButton4:SetPoint("CENTER", 0, 0);
        RaidIconBar_IconButton5:SetPoint("CENTER", -27, 0);
        RaidIconBar_IconButton6:SetPoint("CENTER", 27, 27);
        RaidIconBar_IconButton7:SetPoint("CENTER", 0, 27);
        RaidIconBar_IconButton8:SetPoint("CENTER", -27, 27);

        RaidIconBar_IconButton9:SetPoint("CENTER", 27, -27);
    elseif (index == 6) then
        RaidIconBar_IconFrame:SetWidth(40);
        RaidIconBar_IconFrame:SetHeight(230);

        RaidIconBar_IconButton1:SetPoint("CENTER", 0, 94);
        RaidIconBar_IconButton2:SetPoint("CENTER", 0, 67);
        RaidIconBar_IconButton3:SetPoint("CENTER", 0, 40);
        RaidIconBar_IconButton4:SetPoint("CENTER", 0, 13);
        RaidIconBar_IconButton5:SetPoint("CENTER", 0, -13);
        RaidIconBar_IconButton6:SetPoint("CENTER", 0, -40);
        RaidIconBar_IconButton7:SetPoint("CENTER", 0, -67);
        RaidIconBar_IconButton8:SetPoint("CENTER", 0, -94);

        RaidIconBar_IconButton9:SetPoint("CENTER", 0, -130);
    elseif (index == 7) then
        RaidIconBar_IconFrame:SetWidth(40);
        RaidIconBar_IconFrame:SetHeight(230);

        RaidIconBar_IconButton1:SetPoint("CENTER", 0, -94);
        RaidIconBar_IconButton2:SetPoint("CENTER", 0, -67);
        RaidIconBar_IconButton3:SetPoint("CENTER", 0, -40);
        RaidIconBar_IconButton4:SetPoint("CENTER", 0, -13);
        RaidIconBar_IconButton5:SetPoint("CENTER", 0, 13);
        RaidIconBar_IconButton6:SetPoint("CENTER", 0, 40);
        RaidIconBar_IconButton7:SetPoint("CENTER", 0, 67);
        RaidIconBar_IconButton8:SetPoint("CENTER", 0, 94);

        RaidIconBar_IconButton9:SetPoint("CENTER", 0, 130);
    elseif (index == 8) then
        RaidIconBar_IconFrame:SetWidth(70);
        RaidIconBar_IconFrame:SetHeight(120);

        RaidIconBar_IconButton1:SetPoint("CENTER", -13, -40);
        RaidIconBar_IconButton2:SetPoint("CENTER", -13, -13);
        RaidIconBar_IconButton3:SetPoint("CENTER", -13, 13);
        RaidIconBar_IconButton4:SetPoint("CENTER", -13, 40);
        RaidIconBar_IconButton5:SetPoint("CENTER", 13, -40);
        RaidIconBar_IconButton6:SetPoint("CENTER", 13, -13);
        RaidIconBar_IconButton7:SetPoint("CENTER", 13, 13);
        RaidIconBar_IconButton8:SetPoint("CENTER", 13, 40);

        RaidIconBar_IconButton9:SetPoint("CENTER", 0, 70);
    elseif (index == 9) then
        RaidIconBar_IconFrame:SetWidth(70);
        RaidIconBar_IconFrame:SetHeight(120);

        RaidIconBar_IconButton1:SetPoint("CENTER", -13, 40);
        RaidIconBar_IconButton2:SetPoint("CENTER", -13, 13);
        RaidIconBar_IconButton3:SetPoint("CENTER", -13, -13);
        RaidIconBar_IconButton4:SetPoint("CENTER", -13, -40);
        RaidIconBar_IconButton5:SetPoint("CENTER", 13, 40);
        RaidIconBar_IconButton6:SetPoint("CENTER", 13, 13);
        RaidIconBar_IconButton7:SetPoint("CENTER", 13, -13);
        RaidIconBar_IconButton8:SetPoint("CENTER", 13, -40);

        RaidIconBar_IconButton9:SetPoint("CENTER", 0, -70);
    else
        RaidIconBar_ResetLayout();
    end
end

function RaidIconBar_IconSnapDown()
    icons_posx = RaidIconBar_IconFrame:GetLeft();
    icons_posy = RaidIconBar_IconFrame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale / RaidIconBarData.scale;
    y = y / scale / RaidIconBarData.scale;

    RaidIconBar_IconFrame:ClearAllPoints();
    RaidIconBar_IconFrame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
end

function RaidIconBar_IconSnapUp()
    RaidIconBar_IconFrame:ClearAllPoints();
    RaidIconBar_IconFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", icons_posx, icons_posy);
end

function RaidIconBar_MarkerSnapDown()
    marker_posx = RaidIconBar_MarkerFrame:GetLeft();
    marker_posy = RaidIconBar_MarkerFrame:GetBottom();

    if (marker_posx <= -1000) then
        marker_posx = 300;
        marker_posy = 300;
    end

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    RaidIconBar_MarkerFrame:ClearAllPoints();
    RaidIconBar_MarkerFrame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
end

function RaidIconBar_MarkerSnapUp()
    RaidIconBar_MarkerFrame:ClearAllPoints();
    RaidIconBar_MarkerFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", marker_posx, marker_posy);
end
